/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * /*
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gui.Atmosphere;

import gui.Utils.EntryReal;
import gui.Utils.UnitDependant;
import java.awt.Font;
import java.util.Properties;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import main.atmosphere.Atmosphere;
import main.atmosphere.ConstantAtmosphere;
import main.utils.Units;
import net.miginfocom.swing.MigLayout;

/**
 *
 * @author J. Fourcade
 */
public class ConstantAtmosphereGUI extends JPanel implements UnitDependant {

    // GUI datas
    JTextField rho, T, g;
    JLabel rhoUnit, TUnit, gUnit;

    /**
     *
     * @param atmosphere
     */
    public ConstantAtmosphereGUI(ConstantAtmosphere atmosphere) {

        // Layout
        setLayout(new MigLayout("insets 0 0 0 0",
                "[]15[]15[]5[]20[]15[]15[]5[]"));

        // Panel font
        setFont(new Font("Courier", Font.PLAIN, 12));

        // Density
        add(new JLabel("Density"));
        add(new JLabel(" ρ"));
        rho = new JTextField(String.format("%.6f",
                Units.convertValueFromSI(atmosphere.getDensity(), Units.UnitsList.density)));
        add(rho, "w 80");
        rhoUnit = new JLabel(Units.getLabel(Units.UnitsList.density));
        add(rhoUnit, "wrap");

        // Temperature
        add(new JLabel("Temperature"));
        add(new JLabel(" T"));
        T = new JTextField(String.format("%.2f",
                Units.convertValueFromSI(atmosphere.getTemperature(), Units.UnitsList.temperature)));
        add(T, "w 80");
        TUnit = new JLabel(Units.getLabel(Units.UnitsList.temperature));
        add(TUnit, "wrap");

        // Acceleration
        add(new JLabel("Gravitational acceleration"));
        add(new JLabel(" g"));
        g = new JTextField(String.format("%.2f",
                Units.convertValueFromSI(atmosphere.getAccelerationOfGravity(), Units.UnitsList.acceleration)));
        add(g, "w 80");
        gUnit = new JLabel(Units.getLabel(Units.UnitsList.acceleration));
        add(gUnit, "wrap");

    }

    /**
     *
     */
    @Override
    public void unitsUpdate() {
        EntryReal.unitUpdate(rho, "%.6f", rhoUnit, Units.UnitsList.density);
        EntryReal.unitUpdate(g, "%.2f", gUnit, Units.UnitsList.acceleration);

    }

    /**
     *
     * @param prop
     */
    public void read(Properties prop) {
        EntryReal.readProperties(prop, rho, "ConstantAtmosphere.Density");
        EntryReal.readProperties(prop, T, "ConstantAtmosphere.Temperature");
        EntryReal.readProperties(prop, g, "ConstantAtmosphere.Gravitational_acceleration");
    }

    /**
     *
     * @param prop
     */
    public void write(Properties prop) {
        EntryReal.writeProperties(prop, rho, "ConstantAtmosphere.Density");
        EntryReal.writeProperties(prop, T, "ConstantAtmosphere.Temperature");
        EntryReal.writeProperties(prop, g, "ConstantAtmosphere.Gravitational_acceleration");

    }

    /**
     *
     * @return
     */
    public Atmosphere getAtmosphere() {
        double rhoValue = EntryReal.read(rho, Units.UnitsList.density);
        if (rhoValue <= 0) {
            throw new IllegalArgumentException("Density must be strictly positive");
        }
        double gValue = EntryReal.read(g, Units.UnitsList.acceleration);
        if (gValue <= 0) {
            throw new IllegalArgumentException("Gravitational acceleration must be strictly positive");
        }

        return new ConstantAtmosphere(rhoValue, EntryReal.read(T, Units.UnitsList.temperature),
                gValue);
    }
}
