/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * /*
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gui.Atmosphere;

import gui.Utils.EntryReal;
import gui.Utils.UnitDependant;
import java.awt.Font;
import java.util.Properties;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import main.atmosphere.Atmosphere;
import main.atmosphere.StandardAtmosphere;
import main.utils.Units;
import net.miginfocom.swing.MigLayout;

/**
 *
 * @author J. Fourcade
 */
public class StandardAtmosphereGUI extends JPanel implements UnitDependant {

    // GUI datas
    JTextField h, DT;
    JLabel hUnit, DTUnit;

    /**
     *
     * @param atmosphere
     */
    public StandardAtmosphereGUI(StandardAtmosphere atmosphere) {

        // Layout
        setLayout(new MigLayout("insets 0 0 0 0",
                "[]15[]15[]5[]20[]15[]15[]5[]"));

        // Panel font and title font
        setFont(new Font("Courier", Font.PLAIN, 12));

        // Altitude
        add(new JLabel("Altitude"));
        add(new JLabel(" h"));
        h = new JTextField(String.format("%.1f",
                Units.convertValueFromSI(atmosphere.getAltitude(), Units.UnitsList.length1)));
        add(h, "w 80");
        hUnit = new JLabel(Units.getLabel(Units.UnitsList.length1));
        add(hUnit);

        // Temperature increment
        add(new JLabel("Temperature increment"));
        add(new JLabel(" ΔT"));
        DT = new JTextField(String.format("%.1f",
                Units.convertValueFromSI(atmosphere.getTemperatureIncrement(),
                        Units.UnitsList.temperature)));
        add(DT, "w 80");
        DTUnit = new JLabel(Units.getLabel(Units.UnitsList.temperature));
        add(DTUnit, "wrap");
    }

    /**
     *
     */
    @Override
    public void unitsUpdate() {
        EntryReal.unitUpdate(h, "%.1f", hUnit, Units.UnitsList.length1);
    }

    /**
     *
     * @param prop
     */
    public void read(Properties prop) {
        EntryReal.readProperties(prop, h, "StandardAtmosphere.Altitude");
        EntryReal.readProperties(prop, DT, "StandardAtmosphere.Temperature_increment");
    }

    /**
     *
     * @param prop
     */
    public void write(Properties prop) {
        EntryReal.writeProperties(prop, h, "StandardAtmosphere.Altitude");
        EntryReal.writeProperties(prop, DT, "StandardAtmosphere.Temperature_increment");
    }

    /**
     *
     * @return
     */
    public Atmosphere getAtmosphere() {
        return new StandardAtmosphere(
                EntryReal.read(h, Units.UnitsList.length1),
                EntryReal.read(DT, Units.UnitsList.temperature));
    }
}
