/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * /*
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gui.InducedVelocity;

import java.awt.Font;
import java.util.Properties;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import main.inducedvelocity.ConstantInducedVelocity;
import main.inducedvelocity.MomentumInducedVelocity;
import main.inducedvelocity.ShaydakovInducedVelocity;
import net.miginfocom.swing.MigLayout;

/**
 *
 * @author J. Fourcade
 */
public class InducedVelocityGUI extends JPanel {

    JRadioButton rb1, rb2;
    Font font ;

    /**
     *
     * @param inducedVelocity
     */
    public InducedVelocityGUI(ConstantInducedVelocity inducedVelocity) {

        // Layout
        setLayout(new MigLayout("insets 5 5 5 5"));
        
         // Panel font and title font
        setFont(new Font("Courier", Font.PLAIN, 12)) ;
        font = getFont().deriveFont(Font.BOLD,16) ;
        
        // Title
        JLabel title = new JLabel("Induced velocity :");
        title.setFont(font);
        this.add(title,"wrap");

        // Induced velocity model
        rb1 = new JRadioButton("Shaydakov theory");
        rb1.setSelected(true);
        add(rb1,"split");
        rb1.setActionCommand("Shaydakov");
        rb2 = new JRadioButton("Momentum theory");
        add(rb2,"wrap");
        rb1.setActionCommand("Momentum");
        
        // Group
        ButtonGroup group = new ButtonGroup();
        group.add(rb1);
        group.add(rb2);
        
        if (inducedVelocity instanceof ShaydakovInducedVelocity) {
            rb1.setSelected(true);
            rb2.setSelected(false);
        } else {
            rb1.setSelected(false);
            rb2.setSelected(true);
        }
    }

    /**
     *
     * @param prop
     */
    public void read(Properties prop) {
        String s = prop.getProperty("InducedVelocity.type");
        if (s.equalsIgnoreCase("Shaydakov")) {
            rb1.setSelected(true);
            rb2.setSelected(false);
        } else {
            rb1.setSelected(false);
            rb2.setSelected(true);
        }
    }

    /**
     *
     * @param prop
     */
    public void write(Properties prop) {
        if (rb1.isSelected()) {
            prop.setProperty("InducedVelocity.type", "Shaydakov");
        } else {
            prop.setProperty("InducedVelocity.type", "Momentum");
        }
    }

    /**
     *
     * @return
     */
    public ConstantInducedVelocity getInducedVelocity() {
        if (rb1.isSelected()) {
            return new ShaydakovInducedVelocity();
        } else {
            return new MomentumInducedVelocity();
        }
    }
}
