/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gui.Rotor;

import gui.Utils.EntryReal;
import gui.Utils.UnitDependant;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import main.rotor.GlauertRotor;
import main.utils.Units;
import net.miginfocom.swing.MigLayout;

/**
 * Displau rotor derivatives with respect to rotor disk parameters
 * (u,w,q,p,omega)
 *
 * @author J. Fourcade
 */
public class RotorDerivativesWRTDiscGUI extends JPanel implements UnitDependant {

    // Mode : dimensional or non-dimensional
    private final JRadioButton rb1, rb2;

    // Derivative
    private final JLabel Lu, Lw, Lq, Lp, LW, Lname;
    private final JLabel Du, Dw, Dq, Dp, DW, Dname;
    private final JLabel Tu, Tw, Tq, Tp, TW, Tname;
    private final JLabel Yu, Yw, Yq, Yp, YW, Yname;
    private final JLabel Hu, Hw, Hq, Hp, HW, Hname;
    private final JLabel Qu, Qw, Qq, Qp, QW, Qname;
    private final JLabel a0u, a0w, a0q, a0p, a0W;
    private final JLabel a1u, a1w, a1q, a1p, a1W;
    private final JLabel b1u, b1w, b1q, b1p, b1W;
    private final JLabel a2u, a2w, a2q, a2p, a2W;
    private final JLabel b2u, b2w, b2q, b2p, b2W;
    private final JLabel viu, viw, viq, vip, viW, Viname;

    // Units
    private final JLabel uUnit, wUnit, qUnit, pUnit, WUnit;
    private final JLabel LUnit, YUnit, DUnit, TUnit, HUnit, QUnit,
            a0Unit, a1Unit, b1Unit, a2Unit, b2Unit, viUnit;

    // Derivatives
    private double[][] derivatives;

    // Scale factors
    private double KF, KQ, Kv, Kav;

    // Force Coefficients
    private double CT, CY, CH, CQ, CL, CD;

    class RadioListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            if (e.getActionCommand().equals("Dimensional")) {
                Lname.setText(" L");
                Dname.setText(" D");
                Tname.setText(" T");
                Hname.setText(" H");
                Yname.setText(" Y");
                Qname.setText(" Q");
                Viname.setText(" vi");
                uUnit.setVisible(true);
                wUnit.setVisible(true);
                qUnit.setVisible(true);
                pUnit.setVisible(true);
                WUnit.setVisible(true);
                LUnit.setVisible(true);
                YUnit.setVisible(true);
                DUnit.setVisible(true);
                TUnit.setVisible(true);
                HUnit.setVisible(true);
                QUnit.setVisible(true);
                a0Unit.setVisible(true);
                a1Unit.setVisible(true);
                b1Unit.setVisible(true);
                a2Unit.setVisible(true);
                b2Unit.setVisible(true);
                viUnit.setVisible(true);
            } else {
                Lname.setText(" CL");
                Dname.setText(" CD");
                Tname.setText(" CT");
                Hname.setText(" CH");
                Yname.setText(" CY");
                Qname.setText(" CQ");
                Viname.setText(" λi");
                uUnit.setVisible(false);
                wUnit.setVisible(false);
                qUnit.setVisible(false);
                pUnit.setVisible(false);
                WUnit.setVisible(false);
                LUnit.setVisible(false);
                YUnit.setVisible(false);
                DUnit.setVisible(false);
                TUnit.setVisible(false);
                HUnit.setVisible(false);
                QUnit.setVisible(false);
                a0Unit.setVisible(false);
                a1Unit.setVisible(false);
                a2Unit.setVisible(false);
                b2Unit.setVisible(false);
                b1Unit.setVisible(false);
                viUnit.setVisible(false);
            }

            // Write derivatives
            writeDerivatives();
        }
    }

    /**
     *
     */
    public RotorDerivativesWRTDiscGUI() {

        // Set layout
        setLayout(new MigLayout("hidemode 3, insets 0 0 0 0",
                "[]30[]15[]15[]15[]"));

        // Panel font and title font
        Font panelFont = new Font("Courier", Font.PLAIN, 12);
        setFont(panelFont);
        Font font = new Font("Courier", Font.BOLD, 16);

        // Non-dimensionalisation mode
        rb1 = new JRadioButton("Non-dimensional values");
        rb1.setActionCommand("Non-dimensional");
        rb2 = new JRadioButton("Dimensional values");
        rb2.setActionCommand("Dimensional");
        rb2.setSelected(true);
        add(rb1, "wrap");
        add(rb2, "wrap");

        // Group
        ButtonGroup group = new ButtonGroup();
        group.add(rb1);
        group.add(rb2);

        // Listener
        RadioListener myListener = new RadioListener();
        rb1.addActionListener(myListener);
        rb2.addActionListener(myListener);

        // Units
        uUnit = new JLabel(Units.getLabel(Units.UnitsList.velocity2));
        add(uUnit, "skip 2, alignx center");
        wUnit = new JLabel(Units.getLabel(Units.UnitsList.velocity2));
        add(wUnit, "alignx center");
        qUnit = new JLabel(Units.getLabel(Units.UnitsList.angularVelocity));
        add(qUnit, "alignx center");
        pUnit = new JLabel(Units.getLabel(Units.UnitsList.angularVelocity));
        add(pUnit, "alignx center");
        WUnit = new JLabel(Units.getLabel(Units.UnitsList.rpm));
        add(WUnit, "alignx center, wrap");

        // Derivatives labels
        add(new JLabel("u"), "skip 2, alignx center");
        add(new JLabel("w"), "alignx center");
        add(new JLabel("q"), "alignx center");
        add(new JLabel("p"), "alignx center");
        add(new JLabel("Ω"), "alignx center, wrap");

        // L line
        add(new JLabel("Lift"));
        Lname = new JLabel(" L");
        add(Lname);
        Lu = new JLabel("0.00");
        Lu.setFont(font);
        add(Lu, "alignx right");
        Lw = new JLabel("0.00");
        Lw.setFont(font);
        add(Lw, "alignx right");
        Lq = new JLabel("0.00");
        Lq.setFont(font);
        add(Lq, "alignx right");
        Lp = new JLabel("0.00");
        Lp.setFont(font);
        add(Lp, "alignx right");
        LW = new JLabel("0.00");
        LW.setFont(font);
        add(LW, "alignx right");
        LUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        add(LUnit);

        // D line
        add(new JLabel("Drag"), "newline");
        Dname = new JLabel(" D");
        add(Dname);
        Du = new JLabel("0.00");
        Du.setFont(font);
        add(Du, "alignx right");
        Dw = new JLabel("0.00");
        Dw.setFont(font);
        add(Dw, "alignx right");
        Dq = new JLabel("0.00");
        Dq.setFont(font);
        add(Dq, "alignx right");
        Dp = new JLabel("0.00");
        Dp.setFont(font);
        add(Dp, "alignx right");
        DW = new JLabel("0.00");
        DW.setFont(font);
        add(DW, "alignx right");
        DUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        add(DUnit);

        // T line
        add(new JLabel("Thrust"), "newline");
        Tname = new JLabel(" T");
        add(Tname);
        Tu = new JLabel("0.00");
        Tu.setFont(font);
        add(Tu, "alignx right");
        Tw = new JLabel("0.00");
        Tw.setFont(font);
        add(Tw, "alignx right");
        Tq = new JLabel("0.00");
        Tq.setFont(font);
        add(Tq, "alignx right");
        Tp = new JLabel("0.00");
        Tp.setFont(font);
        add(Tp, "alignx right");
        TW = new JLabel("0.00");
        add(TW, "alignx right");
        TW.setFont(font);
        TUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        add(TUnit);

        // Y line
        add(new JLabel("Lateral force"), "newline");
        Yname = new JLabel(" Y");
        add(Yname);
        Yu = new JLabel("0.00");
        Yu.setFont(font);
        add(Yu, "alignx right");
        Yw = new JLabel("0.00");
        Yw.setFont(font);
        add(Yw, "alignx right");
        Yq = new JLabel("0.00");
        Yq.setFont(font);
        add(Yq, "alignx right");
        Yp = new JLabel("0.00");
        Yp.setFont(font);
        add(Yp, "alignx right");
        YW = new JLabel("0.00");
        YW.setFont(font);
        add(YW, "alignx right");
        YUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        add(YUnit);

        // H line
        add(new JLabel("Rear force"), "newline");
        Hname = new JLabel(" H");
        add(Hname);
        Hu = new JLabel("0.00");
        Hu.setFont(font);
        add(Hu, "alignx right");
        Hw = new JLabel("0.00");
        Hw.setFont(font);
        add(Hw, "alignx right");
        Hq = new JLabel("0.00");
        Hq.setFont(font);
        add(Hq, "alignx right");
        Hp = new JLabel("0.00");
        Hp.setFont(font);
        add(Hp, "alignx right");
        HW = new JLabel("0.00");
        HW.setFont(font);
        add(HW, "alignx right");
        HUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        add(HUnit);

        // Q line
        add(new JLabel("Torque"), "newline");
        Qname = new JLabel(" Q");
        add(Qname);
        Qu = new JLabel("0.00");
        Qu.setFont(font);
        add(Qu, "alignx right");
        Qw = new JLabel("0.00");
        Qw.setFont(font);
        add(Qw, "alignx right");
        Qq = new JLabel("0.00");
        Qq.setFont(font);
        add(Qq, "alignx right");
        Qp = new JLabel("0.00");
        Qp.setFont(font);
        add(Qp, "alignx right");
        QW = new JLabel("0.00");
        QW.setFont(font);
        add(QW, "alignx right");
        QUnit = new JLabel(Units.getLabel(Units.UnitsList.moment));
        add(QUnit);

        // a0 line
        add(new JLabel("Conning angle"), "newline");
        add(new JLabel(" a0"));
        a0u = new JLabel("0.00");
        a0u.setFont(font);
        add(a0u, "alignx right");
        a0w = new JLabel("0.00");
        a0w.setFont(font);
        add(a0w, "alignx right");
        a0q = new JLabel("0.00");
        a0q.setFont(font);
        add(a0q, "alignx right");
        a0p = new JLabel("0.00");
        a0p.setFont(font);
        add(a0p, "alignx right");
        a0W = new JLabel("0.00");
        a0W.setFont(font);
        add(a0W, "alignx right");
        a0Unit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        add(a0Unit);

        // a1 line
        add(new JLabel("Longitudinal flapping"), "newline");
        add(new JLabel(" a1"));
        a1u = new JLabel("0.00");
        a1u.setFont(font);
        add(a1u, "alignx right");
        a1w = new JLabel("0.00");
        a1w.setFont(font);
        add(a1w, "alignx right");
        a1q = new JLabel("0.00");
        a1q.setFont(font);
        add(a1q, "alignx right");
        a1p = new JLabel("0.00");
        a1p.setFont(font);
        add(a1p, "alignx right");
        a1W = new JLabel("0.00");
        a1W.setFont(font);
        add(a1W, "alignx right");
        a1Unit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        add(a1Unit);

        // b1 line
        add(new JLabel("Lateral flapping"), "newline");
        add(new JLabel(" b1"));
        b1u = new JLabel("0.00");
        b1u.setFont(font);
        add(b1u, "alignx right");
        b1w = new JLabel("0.00");
        b1w.setFont(font);
        add(b1w, "alignx right");
        b1q = new JLabel("0.00");
        b1q.setFont(font);
        add(b1q, "alignx right");
        b1p = new JLabel("0.00");
        b1p.setFont(font);
        add(b1p, "alignx right");
        b1W = new JLabel("0.00");
        b1W.setFont(font);
        add(b1W, "alignx right");
        b1Unit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        add(b1Unit);

        // a2 line
        add(new JLabel("2nd Longitudinal flapping"), "newline");
        add(new JLabel(" a2"));
        a2u = new JLabel("0.00");
        a2u.setFont(font);
        add(a2u, "alignx right");
        a2w = new JLabel("0.00");
        a2w.setFont(font);
        add(a2w, "alignx right");
        a2q = new JLabel("0.00");
        a2q.setFont(font);
        add(a2q, "alignx right");
        a2p = new JLabel("0.00");
        a2p.setFont(font);
        add(a2p, "alignx right");
        a2W = new JLabel("0.00");
        a2W.setFont(font);
        add(a2W, "alignx right");
        a2Unit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        add(a2Unit);

        // b2 line
        add(new JLabel("2nd Lateral flapping"), "newline");
        add(new JLabel(" b2"));
        b2u = new JLabel("0.00");
        b2u.setFont(font);
        add(b2u, "alignx right");
        b2w = new JLabel("0.00");
        b2w.setFont(font);
        add(b2w, "alignx right");
        b2q = new JLabel("0.00");
        b2q.setFont(font);
        add(b2q, "alignx right");
        b2p = new JLabel("0.00");
        b2p.setFont(font);
        add(b2p, "alignx right");
        b2W = new JLabel("0.00");
        b2W.setFont(font);
        add(b2W, "alignx right");
        b2Unit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        add(b2Unit);

        // vi line
        add(new JLabel("Induced velocity"), "newline");
        Viname = new JLabel(" vi");
        add(Viname);
        viu = new JLabel("0.00");
        viu.setFont(font);
        add(viu, "alignx right");
        viw = new JLabel("0.00");
        viw.setFont(font);
        add(viw, "alignx right");
        viq = new JLabel("0.00");
        viq.setFont(font);
        add(viq, "alignx right");
        vip = new JLabel("0.00");
        vip.setFont(font);
        add(vip, "alignx right");
        viW = new JLabel("0.00");
        viW.setFont(font);
        add(viW, "alignx right");
        viUnit = new JLabel(Units.getLabel(Units.UnitsList.velocity2));
        add(viUnit);
    }

    /**
     *
     * @param rotor
     */
    public void setValues(GlauertRotor rotor) {

        // Set
        derivatives = rotor.getDerivativesWRdisc();

        // Get scale factors
        KF = rotor.getForceScaleFactor();
        KQ = rotor.getMomentScaleFactor();
        Kv = rotor.getVelocityScaleFactor();
        Kav = rotor.getAngularVelocityScaleFactor();

        // Get force coefficients
        CT = rotor.getThrust() / KF;
        CY = rotor.getLateralForce() / KF;
        CH = rotor.getTotalRearForce() / KF;
        CQ = rotor.getTotalTorque() / KQ;
        CL = rotor.getLift() / KF;
        CD = rotor.getDrag() / KF;

        // Write derivatives
        writeDerivatives();
    }

    public void setMode(boolean dimensional) {
        if (dimensional) {
            uUnit.setVisible(true);
            wUnit.setVisible(true);
            qUnit.setVisible(true);
            pUnit.setVisible(true);
            WUnit.setVisible(true);
            LUnit.setVisible(true);
            YUnit.setVisible(true);
            DUnit.setVisible(true);
            TUnit.setVisible(true);
            HUnit.setVisible(true);
            QUnit.setVisible(true);
            a0Unit.setVisible(true);
            a1Unit.setVisible(true);
            b1Unit.setVisible(true);
            a2Unit.setVisible(true);
            b2Unit.setVisible(true);
            viUnit.setVisible(true);
        } else {
            uUnit.setVisible(false);
            wUnit.setVisible(false);
            qUnit.setVisible(false);
            pUnit.setVisible(false);
            WUnit.setVisible(false);
            LUnit.setVisible(false);
            YUnit.setVisible(false);
            DUnit.setVisible(false);
            TUnit.setVisible(false);
            HUnit.setVisible(false);
            QUnit.setVisible(false);
            a0Unit.setVisible(false);
            a1Unit.setVisible(false);
            a2Unit.setVisible(false);
            b2Unit.setVisible(false);
            b1Unit.setVisible(false);
            viUnit.setVisible(false);
        }
    }

    @Override
    public void unitsUpdate() {

        // Derivatives units
        uUnit.setText(Units.getLabel(Units.UnitsList.velocity2));
        wUnit.setText(Units.getLabel(Units.UnitsList.velocity2));
        LUnit.setText(Units.getLabel(Units.UnitsList.force));
        DUnit.setText(Units.getLabel(Units.UnitsList.force));
        TUnit.setText(Units.getLabel(Units.UnitsList.force));
        YUnit.setText(Units.getLabel(Units.UnitsList.force));
        HUnit.setText(Units.getLabel(Units.UnitsList.force));
        QUnit.setText(Units.getLabel(Units.UnitsList.moment));
        viUnit.setText(Units.getLabel(Units.UnitsList.velocity2));

        // Values
        writeDerivatives();

    }

    private void writeDerivatives() {
        // Write derivatives
        if (derivatives != null) {
            if (rb1.isSelected()) {
                EntryReal.set(Lu, derivatives[0][0] * Kv / KF, "%6.2e");
                EntryReal.set(Du, derivatives[1][0] * Kv / KF, "%6.2e");
                EntryReal.set(Tu, derivatives[2][0] * Kv / KF, "%6.2e");
                EntryReal.set(Yu, derivatives[3][0] * Kv / KF, "%6.2e");
                EntryReal.set(Hu, derivatives[4][0] * Kv / KF, "%6.2e");
                EntryReal.set(Qu, derivatives[5][0] * Kv / KQ, "%6.2e");
                EntryReal.set(a0u, derivatives[6][0] * Kv, "%6.2e");
                EntryReal.set(a1u, derivatives[7][0] * Kv, "%6.2e");
                EntryReal.set(b1u, derivatives[8][0] * Kv, "%6.2e");
                EntryReal.set(a2u, derivatives[9][0] * Kv, "%6.2e");
                EntryReal.set(b2u, derivatives[10][0] * Kv, "%6.2e");
                EntryReal.set(viu, derivatives[11][0], "%6.2e");

                EntryReal.set(Lw, derivatives[0][1] * Kv / KF, "%6.2e");
                EntryReal.set(Dw, derivatives[1][1] * Kv / KF, "%6.2e");
                EntryReal.set(Tw, derivatives[2][1] * Kv / KF, "%6.2e");
                EntryReal.set(Yw, derivatives[3][1] * Kv / KF, "%6.2e");
                EntryReal.set(Hw, derivatives[4][1] * Kv / KF, "%6.2e");
                EntryReal.set(Qw, derivatives[5][1] * Kv / KQ, "%6.2e");
                EntryReal.set(a0w, derivatives[6][1] * Kv, "%6.2e");
                EntryReal.set(a1w, derivatives[7][1] * Kv, "%6.2e");
                EntryReal.set(b1w, derivatives[8][1] * Kv, "%6.2e");
                EntryReal.set(a2w, derivatives[9][1] * Kv, "%6.2e");
                EntryReal.set(b2w, derivatives[10][1] * Kv, "%6.2e");
                EntryReal.set(viw, derivatives[11][1], "%6.2e");

                EntryReal.set(Lq, derivatives[0][2] * Kav / KF, "%6.2e");
                EntryReal.set(Dq, derivatives[1][2] * Kav / KF, "%6.2e");
                EntryReal.set(Tq, derivatives[2][2] * Kav / KF, "%6.2e");
                EntryReal.set(Yq, derivatives[3][2] * Kav / KF, "%6.2e");
                EntryReal.set(Hq, derivatives[4][2] * Kav / KF, "%6.2e");
                EntryReal.set(Qq, derivatives[5][2] * Kav / KQ, "%6.2e");
                EntryReal.set(a0q, derivatives[6][2] * Kav, "%6.2e");
                EntryReal.set(a1q, derivatives[7][2] * Kav, "%6.2e");
                EntryReal.set(b1q, derivatives[8][2] * Kav, "%6.2e");
                EntryReal.set(a2q, derivatives[9][2] * Kav, "%6.2e");
                EntryReal.set(b2q, derivatives[10][2] * Kav, "%6.2e");
                EntryReal.set(viq, derivatives[11][2] * Kav / Kv, "%6.2e");

                EntryReal.set(Lp, derivatives[0][3] * Kav / KF, "%6.2e");
                EntryReal.set(Dp, derivatives[1][3] * Kav / KF, "%6.2e");
                EntryReal.set(Tp, derivatives[2][3] * Kav / KF, "%6.2e");
                EntryReal.set(Yp, derivatives[3][3] * Kav / KF, "%6.2e");
                EntryReal.set(Hp, derivatives[4][3] * Kav / KF, "%6.2e");
                EntryReal.set(Qp, derivatives[5][3] * Kav / KQ, "%6.2e");
                EntryReal.set(a0p, derivatives[6][3] * Kav, "%6.2e");
                EntryReal.set(a1p, derivatives[7][3] * Kav, "%6.2e");
                EntryReal.set(b1p, derivatives[8][3] * Kav, "%6.2e");
                EntryReal.set(a2p, derivatives[9][3] * Kav, "%6.2e");
                EntryReal.set(b2p, derivatives[10][3] * Kav, "%6.2e");
                EntryReal.set(vip, derivatives[11][3] * Kav / Kv, "%6.2e");

                EntryReal.set(LW, derivatives[0][4] * Kav / KF - 2 * CL, "%6.2e");
                EntryReal.set(DW, derivatives[1][4] * Kav / KF - 2 * CD, "%6.2e");
                EntryReal.set(TW, derivatives[2][4] * Kav / KF - 2 * CT, "%6.2e");
                EntryReal.set(YW, derivatives[3][4] * Kav / KF - 2 * CY, "%6.2e");
                EntryReal.set(HW, derivatives[4][4] * Kav / KF - 2 * CH, "%6.2e");
                EntryReal.set(QW, derivatives[5][4] * Kav / KQ - 2 * CQ, "%6.2e");
                EntryReal.set(a0W, derivatives[6][4] * Kav, "%6.2e");
                EntryReal.set(a1W, derivatives[7][4] * Kav, "%6.2e");
                EntryReal.set(b1W, derivatives[8][4] * Kav, "%6.2e");
                EntryReal.set(a2W, derivatives[9][4] * Kav, "%6.2e");
                EntryReal.set(b2W, derivatives[10][4] * Kav, "%6.2e");
                EntryReal.set(viW, derivatives[11][4] * Kav / Kv, "%6.2e");
            } else {
                EntryReal.set(Lu, derivatives[0][0], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Du, derivatives[1][0], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Tu, derivatives[2][0], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Yu, derivatives[3][0], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Hu, derivatives[4][0], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Qu, derivatives[5][0], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.velocity2);
                EntryReal.set(a0u, derivatives[6][0], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.velocity2);
                EntryReal.set(a1u, derivatives[7][0], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.velocity2);
                EntryReal.set(b1u, derivatives[8][0], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.velocity2);
                EntryReal.set(a2u, derivatives[9][0], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.velocity2);
                EntryReal.set(b2u, derivatives[10][0], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.velocity2);
                EntryReal.set(viu, derivatives[11][0], "%6.2e", Units.UnitsList.velocity2,
                        Units.UnitsList.velocity2);

                EntryReal.set(Lw, derivatives[0][1], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Dw, derivatives[1][1], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Tw, derivatives[2][1], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Yw, derivatives[3][1], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Hw, derivatives[4][1], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Qw, derivatives[5][1], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.velocity2);
                EntryReal.set(a0w, derivatives[6][1], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.velocity2);
                EntryReal.set(a1w, derivatives[7][1], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.velocity2);
                EntryReal.set(b1w, derivatives[8][1], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.velocity2);
                EntryReal.set(a2w, derivatives[9][1], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.velocity2);
                EntryReal.set(b2w, derivatives[10][1], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.velocity2);
                EntryReal.set(viw, derivatives[11][1], "%6.2e", Units.UnitsList.velocity2,
                        Units.UnitsList.velocity2);

                EntryReal.set(Lq, derivatives[0][2], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Dq, derivatives[1][2], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Tq, derivatives[2][2], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Yq, derivatives[3][2], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Hq, derivatives[4][2], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Qq, derivatives[5][2], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(a0q, derivatives[6][2], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(a1q, derivatives[7][2], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(b1q, derivatives[8][2], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(a2q, derivatives[9][2], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(b2q, derivatives[10][2], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(viq, derivatives[11][2], "%6.2e", Units.UnitsList.velocity2,
                        Units.UnitsList.angularVelocity);

                EntryReal.set(Lp, derivatives[0][3], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Dp, derivatives[1][3], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Tp, derivatives[2][3], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Yp, derivatives[3][3], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Hp, derivatives[4][3], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Qp, derivatives[5][3], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(a0p, derivatives[6][3], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(a1p, derivatives[7][3], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(b1p, derivatives[8][3], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(a2p, derivatives[9][3], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(b2p, derivatives[10][3], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(vip, derivatives[11][3], "%6.2e", Units.UnitsList.velocity2,
                        Units.UnitsList.angularVelocity);

                EntryReal.set(LW, derivatives[0][4], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.rpm);
                EntryReal.set(DW, derivatives[1][4], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.rpm);
                EntryReal.set(YW, derivatives[2][4], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.rpm);
                EntryReal.set(TW, derivatives[3][4], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(HW, derivatives[4][4], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(QW, derivatives[5][4], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(a0W, derivatives[6][4], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(a1W, derivatives[7][4], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(b1W, derivatives[8][4], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(a2W, derivatives[9][4], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(b2W, derivatives[10][4], "%6.2e", Units.UnitsList.angle,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(viW, derivatives[11][4], "%6.2e", Units.UnitsList.velocity2,
                        Units.UnitsList.angularVelocity);
            }
        }
    }
}
