/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
/*
03/10/15 : Conversion factor of derivatives[*][2] and derivatives[*][3] in case 
of normalized values was erroneous.
*/
package gui.Rotor;

import gui.Utils.EntryReal;
import gui.Utils.UnitDependant;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import main.rotor.Rotor;
import main.utils.Units;
import net.miginfocom.swing.MigLayout;
import org.apache.commons.math3.geometry.euclidean.threed.Vector3D;

/**
 *
 * @author J. Fourcade
 */
public class RotorDerivativesWRTFuselageGUI extends JPanel implements UnitDependant {

    // Mode : dimensional or non-dimensional
    private final JRadioButton rb1, rb2;

    // Derivative panel
    JPanel derivativesPanel;

    // Fuselage data
    JTextField alphaF, XH, YH, ZH, m, Ixx, Iyy, Izz, IR;
    JLabel alphaFUnit, XYZUnit, mUnit, moiUnit, IRUnit;

    // Derivative
    private final JLabel Xu, Xw, Xq, Xp, XW;
    private final JLabel Yu, Yw, Yq, Yp, YW;
    private final JLabel Zu, Zw, Zq, Zp, ZW;
    private final JLabel Lu, Lw, Lq, Lp, LW;
    private final JLabel Mu, Mw, Mq, Mp, MW;
    private final JLabel Nu, Nw, Nq, Np, NW;
    private final JLabel Qu, Qw, Qq, Qp, QW;

    // Units
    private final JLabel uUnit, wUnit, qUnit, pUnit, WUnit;
    private final JLabel XUnit, YUnit, ZUnit, LUnit, MUnit, NUnit, QUnit;

    // Derivatives
    private double[][] derivatives;

    class RadioListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            if (e.getActionCommand().equals("Normalized")) {
                XUnit.setText(Units.getLabel(Units.UnitsList.acceleration));
                YUnit.setText(Units.getLabel(Units.UnitsList.acceleration));
                ZUnit.setText(Units.getLabel(Units.UnitsList.acceleration));
                LUnit.setText(Units.getLabel(Units.UnitsList.radianAngularAcceleration));
                MUnit.setText(Units.getLabel(Units.UnitsList.radianAngularAcceleration));
                NUnit.setText(Units.getLabel(Units.UnitsList.radianAngularAcceleration));
                QUnit.setText(Units.getLabel(Units.UnitsList.rpmAcceleration));
                qUnit.setText(Units.getLabel(Units.UnitsList.radianAngularVelocity));
                pUnit.setText(Units.getLabel(Units.UnitsList.radianAngularVelocity));
            } else {
                XUnit.setText(Units.getLabel(Units.UnitsList.force));
                YUnit.setText(Units.getLabel(Units.UnitsList.force));
                ZUnit.setText(Units.getLabel(Units.UnitsList.force));
                LUnit.setText(Units.getLabel(Units.UnitsList.moment));
                MUnit.setText(Units.getLabel(Units.UnitsList.moment));
                NUnit.setText(Units.getLabel(Units.UnitsList.moment));
                QUnit.setText(Units.getLabel(Units.UnitsList.moment));
                qUnit.setText(Units.getLabel(Units.UnitsList.angularVelocity));
                pUnit.setText(Units.getLabel(Units.UnitsList.angularVelocity));
            }

            // Write derivatives
            writeDerivatives();
        }
    }

    /**
     *
     */
    public RotorDerivativesWRTFuselageGUI() {

        // Set layout
        setLayout(new MigLayout("insets 0 0 0 0"));

        // Panels
        derivativesPanel = new JPanel(new MigLayout("hidemode 3, insets 0 0 0 0",
                "[]30[]15[]15[]15[]"));

        // Panel font and title font
        Font panelFont = new Font("Courier", Font.PLAIN, 12);
        setFont(panelFont);
        Font font = new Font("Courier", Font.BOLD, 16);

        // Non-dimensionalisation mode
        rb1 = new JRadioButton("Normalized values");
        rb1.setActionCommand("Normalized");
        rb2 = new JRadioButton("Non-normalized values");
        rb2.setActionCommand("Non-normalized");
        rb2.setSelected(true);
        add(rb1, "wrap");
        add(rb2, "wrap 15px");

        // Group
        ButtonGroup group = new ButtonGroup();
        group.add(rb1);
        group.add(rb2);

        // Listener
        RadioListener myListener = new RadioListener();
        rb1.addActionListener(myListener);
        rb2.addActionListener(myListener);

        // Fuselage incidence
        add(new JLabel("Fuselage incidence"));
        add(new JLabel(" αF"));
        alphaF = new JTextField("0.0");
        add(alphaF, "w 80");
        alphaFUnit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        add(alphaFUnit, "wrap");

        // Hub position
        add(new JLabel("Hub position"));
        add(new JLabel(" X"));
        XH = new JTextField("0.0");
        add(XH, "w 80");
        add(new JLabel(" Y"));
        YH = new JTextField("0.0");
        add(YH, "w 80");
        add(new JLabel(" Z"), "w 40");
        ZH = new JTextField("0.0");
        add(ZH, "w 80");
        XYZUnit = new JLabel(Units.getLabel(Units.UnitsList.length1));
        add(XYZUnit, "wrap 15px");

        // Gyro mass
        add(new JLabel("Gross mass"));
        add(new JLabel(" m"));
        m = new JTextField("0.0");
        add(m, "w 80");
        mUnit = new JLabel(Units.getLabel(Units.UnitsList.mass));
        add(mUnit, "wrap");

        // Rotor moment of inertia
        add(new JLabel("Rotor moment of inertia"));
        add(new JLabel(" IR"));
        IR = new JTextField("0.0");
        add(IR, "w 80");
        IRUnit = new JLabel(Units.getLabel(Units.UnitsList.momentOfInertia));
        add(IRUnit, "wrap");

        // Fuselage moment of inertia
        add(new JLabel("Fuselage moment of inertia"));
        add(new JLabel(" Ixx"));
        Ixx = new JTextField("0.0");
        add(Ixx, "w 80");
        add(new JLabel(" Iyy"));
        Iyy = new JTextField("0.0");
        add(Iyy, "w 80");
        add(new JLabel(" Izz"));
        Izz = new JTextField("0.0");
        add(Izz, "w 80");
        moiUnit = new JLabel(Units.getLabel(Units.UnitsList.momentOfInertia));
        add(moiUnit, "wrap 15px");

        // Units
        uUnit = new JLabel(Units.getLabel(Units.UnitsList.velocity2));
        derivativesPanel.add(uUnit, "skip 2, alignx center");
        wUnit = new JLabel(Units.getLabel(Units.UnitsList.velocity2));
        derivativesPanel.add(wUnit, "alignx center");
        qUnit = new JLabel(Units.getLabel(Units.UnitsList.angularVelocity));
        derivativesPanel.add(qUnit, "alignx center");
        pUnit = new JLabel(Units.getLabel(Units.UnitsList.angularVelocity));
        derivativesPanel.add(pUnit, "alignx center");
        WUnit = new JLabel(Units.getLabel(Units.UnitsList.rpm));
        derivativesPanel.add(WUnit, "alignx center, wrap");

        // Derivatives labels
        derivativesPanel.add(new JLabel("u'"), "skip 2, alignx center");
        derivativesPanel.add(new JLabel("w'"), "alignx center");
        derivativesPanel.add(new JLabel("q"), "alignx center");
        derivativesPanel.add(new JLabel("p'"), "alignx center");
        derivativesPanel.add(new JLabel("Ω"), "alignx center, wrap");

        // X line
        derivativesPanel.add(new JLabel("Roll axis force"), "newline");
        derivativesPanel.add(new JLabel(" X"));
        Xu = new JLabel("0.00");
        Xu.setFont(font);
        derivativesPanel.add(Xu, "alignx right");
        Xw = new JLabel("0.00");
        Xw.setFont(font);
        derivativesPanel.add(Xw, "alignx right");
        Xq = new JLabel("0.00");
        Xq.setFont(font);
        derivativesPanel.add(Xq, "alignx right");
        Xp = new JLabel("0.00");
        Xp.setFont(font);
        derivativesPanel.add(Xp, "alignx right");
        XW = new JLabel("0.00");
        XW.setFont(font);
        derivativesPanel.add(XW, "alignx right");
        XUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        derivativesPanel.add(XUnit);

        // Y line
        derivativesPanel.add(new JLabel("Pitch axis force"), "newline");
        derivativesPanel.add(new JLabel(" Y"));
        Yu = new JLabel("0.00");
        Yu.setFont(font);
        derivativesPanel.add(Yu, "alignx right");
        Yw = new JLabel("0.00");
        Yw.setFont(font);
        derivativesPanel.add(Yw, "alignx right");
        Yq = new JLabel("0.00");
        Yq.setFont(font);
        derivativesPanel.add(Yq, "alignx right");
        Yp = new JLabel("0.00");
        Yp.setFont(font);
        derivativesPanel.add(Yp, "alignx right");
        YW = new JLabel("0.00");
        YW.setFont(font);
        derivativesPanel.add(YW, "alignx right");
        YUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        derivativesPanel.add(YUnit);

        // Z line
        derivativesPanel.add(new JLabel("Yaw axis force"), "newline");
        derivativesPanel.add(new JLabel(" Z"));
        Zu = new JLabel("0.00");
        Zu.setFont(font);
        derivativesPanel.add(Zu, "alignx right");
        Zw = new JLabel("0.00");
        Zw.setFont(font);
        derivativesPanel.add(Zw, "alignx right");
        Zq = new JLabel("0.00");
        Zq.setFont(font);
        derivativesPanel.add(Zq, "alignx right");
        Zp = new JLabel("0.00");
        Zp.setFont(font);
        derivativesPanel.add(Zp, "alignx right");
        ZW = new JLabel("0.00");
        ZW.setFont(font);
        derivativesPanel.add(ZW, "alignx right");
        ZUnit = new JLabel(Units.getLabel(Units.UnitsList.force));
        derivativesPanel.add(ZUnit);

        // L line
        derivativesPanel.add(new JLabel("Roll moment"), "newline");
        derivativesPanel.add(new JLabel(" L"));
        Lu = new JLabel("0.00");
        Lu.setFont(font);
        derivativesPanel.add(Lu, "alignx right");
        Lw = new JLabel("0.00");
        Lw.setFont(font);
        derivativesPanel.add(Lw, "alignx right");
        Lq = new JLabel("0.00");
        Lq.setFont(font);
        derivativesPanel.add(Lq, "alignx right");
        Lp = new JLabel("0.00");
        Lp.setFont(font);
        derivativesPanel.add(Lp, "alignx right");
        LW = new JLabel("0.00");
        LW.setFont(font);
        derivativesPanel.add(LW, "alignx right");
        LUnit = new JLabel(Units.getLabel(Units.UnitsList.moment));
        derivativesPanel.add(LUnit);

        // M line
        derivativesPanel.add(new JLabel("Pitch moment"), "newline");
        derivativesPanel.add(new JLabel(" M"));
        Mu = new JLabel("0.00");
        Mu.setFont(font);
        derivativesPanel.add(Mu, "alignx right");
        Mw = new JLabel("0.00");
        Mw.setFont(font);
        derivativesPanel.add(Mw, "alignx right");
        Mq = new JLabel("0.00");
        Mq.setFont(font);
        derivativesPanel.add(Mq, "alignx right");
        Mp = new JLabel("0.00");
        Mp.setFont(font);
        derivativesPanel.add(Mp, "alignx right");
        MW = new JLabel("0.00");
        MW.setFont(font);
        derivativesPanel.add(MW, "alignx right");
        MUnit = new JLabel(Units.getLabel(Units.UnitsList.moment));
        derivativesPanel.add(MUnit);

        // N line
        derivativesPanel.add(new JLabel("Yaw moment"), "newline");
        derivativesPanel.add(new JLabel(" N"));
        Nu = new JLabel("0.00");
        Nu.setFont(font);
        derivativesPanel.add(Nu, "alignx right");
        Nw = new JLabel("0.00");
        Nw.setFont(font);
        derivativesPanel.add(Nw, "alignx right");
        Nq = new JLabel("0.00");
        Nq.setFont(font);
        derivativesPanel.add(Nq, "alignx right");
        Np = new JLabel("0.00");
        Np.setFont(font);
        derivativesPanel.add(Np, "alignx right");
        NW = new JLabel("0.00");
        NW.setFont(font);
        derivativesPanel.add(NW, "alignx right");
        NUnit = new JLabel(Units.getLabel(Units.UnitsList.moment));
        derivativesPanel.add(NUnit);

        // Q line
        derivativesPanel.add(new JLabel("Rotor torque"), "newline");
        derivativesPanel.add(new JLabel(" Q"));
        Qu = new JLabel("0.00");
        Qu.setFont(font);
        derivativesPanel.add(Qu, "alignx right");
        Qw = new JLabel("0.00");
        Qw.setFont(font);
        derivativesPanel.add(Qw, "alignx right");
        Qq = new JLabel("0.00");
        Qq.setFont(font);
        derivativesPanel.add(Qq, "alignx right");
        Qp = new JLabel("0.00");
        Qp.setFont(font);
        derivativesPanel.add(Qp, "alignx right");
        QW = new JLabel("0.00");
        QW.setFont(font);
        derivativesPanel.add(QW, "alignx right");
        QUnit = new JLabel(Units.getLabel(Units.UnitsList.moment));
        derivativesPanel.add(QUnit);

        // Add panel$
        add(derivativesPanel, "spanx 10");
    }

    /**
     *
     * @param rotor
     */
    public void setValues(Rotor rotor) {

        // get derivatives
        derivatives = rotor.getDerivativesWRfuselage();

        // Write derivatives
        writeDerivatives();
    }

    public double getFuselageIncidence() {
        final double alphaFValue = EntryReal.read(alphaF, Units.UnitsList.angle);
        return alphaFValue;
    }

    public Vector3D getHubPosition() {
        final Vector3D hubPosition = new Vector3D(
                EntryReal.read(XH, Units.UnitsList.length1),
                EntryReal.read(YH, Units.UnitsList.length1),
                EntryReal.read(ZH, Units.UnitsList.length1));
        return hubPosition;
    }

    @Override
    public void unitsUpdate() {
        // Derivatives units
        uUnit.setText(Units.getLabel(Units.UnitsList.velocity2));
        wUnit.setText(Units.getLabel(Units.UnitsList.velocity2));
        if (rb2.isSelected()) {
            XUnit.setText(Units.getLabel(Units.UnitsList.force));
            YUnit.setText(Units.getLabel(Units.UnitsList.force));
            ZUnit.setText(Units.getLabel(Units.UnitsList.force));
            LUnit.setText(Units.getLabel(Units.UnitsList.moment));
            MUnit.setText(Units.getLabel(Units.UnitsList.moment));
            NUnit.setText(Units.getLabel(Units.UnitsList.moment));
            QUnit.setText(Units.getLabel(Units.UnitsList.moment));
            qUnit.setText(Units.getLabel(Units.UnitsList.angularVelocity));
            pUnit.setText(Units.getLabel(Units.UnitsList.angularVelocity));
        } else {
            XUnit.setText(Units.getLabel(Units.UnitsList.acceleration));
            YUnit.setText(Units.getLabel(Units.UnitsList.acceleration));
            ZUnit.setText(Units.getLabel(Units.UnitsList.acceleration));
            LUnit.setText(Units.getLabel(Units.UnitsList.radianAngularAcceleration));
            MUnit.setText(Units.getLabel(Units.UnitsList.radianAngularAcceleration));
            NUnit.setText(Units.getLabel(Units.UnitsList.radianAngularAcceleration));
            QUnit.setText(Units.getLabel(Units.UnitsList.rpmAcceleration));
            qUnit.setText(Units.getLabel(Units.UnitsList.radianAngularVelocity));
            pUnit.setText(Units.getLabel(Units.UnitsList.radianAngularVelocity));
        }

        EntryReal.unitUpdate(m, "%.2f", mUnit, Units.UnitsList.mass);
        EntryReal.unitUpdate(Ixx, "%.2f", moiUnit, Units.UnitsList.momentOfInertia);
        EntryReal.unitUpdate(Iyy, "%.2f", moiUnit, Units.UnitsList.momentOfInertia);
        EntryReal.unitUpdate(Izz, "%.2f", moiUnit, Units.UnitsList.momentOfInertia);
        EntryReal.unitUpdate(IR, "%.2f", IRUnit, Units.UnitsList.momentOfInertia);
        EntryReal.unitUpdate(XH, "%.2f", XYZUnit, Units.UnitsList.length1);
        EntryReal.unitUpdate(YH, "%.2f", XYZUnit, Units.UnitsList.length1);
        EntryReal.unitUpdate(ZH, "%.2f", XYZUnit, Units.UnitsList.length1);

        // Values
        writeDerivatives();
    }

    private void writeDerivatives() {
        // Write derivatives
        if (derivatives != null) {
            if (rb1.isSelected()) {
                final double mValue = EntryReal.read(m, Units.UnitsList.mass);
                final double IRValue = EntryReal.read(IR, Units.UnitsList.momentOfInertia);
                final double IxxValue = EntryReal.read(Ixx, Units.UnitsList.momentOfInertia);
                final double IyyValue = EntryReal.read(Iyy, Units.UnitsList.momentOfInertia);
                final double IzzValue = EntryReal.read(Izz, Units.UnitsList.momentOfInertia);
                EntryReal.set(Xu, derivatives[0][0] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Yu, derivatives[1][0] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Zu, derivatives[2][0] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Lu, derivatives[3][0] / IxxValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Mu, derivatives[4][0] / IyyValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Nu, derivatives[5][0] / IzzValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Qu, derivatives[6][0] / IRValue, "%6.2e", Units.UnitsList.rpmAcceleration,
                        Units.UnitsList.velocity2);

                EntryReal.set(Xw, derivatives[0][1] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Yw, derivatives[1][1] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Zw, derivatives[2][1] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Lw, derivatives[3][1] / IxxValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Mw, derivatives[4][1] / IyyValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Nw, derivatives[5][1] / IzzValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.velocity2);
                EntryReal.set(Qw, derivatives[6][1] / IRValue, "%6.2e", Units.UnitsList.rpmAcceleration,
                        Units.UnitsList.velocity2);

                EntryReal.set(Xq, derivatives[0][2] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.radianAngularVelocity);
                EntryReal.set(Yq, derivatives[1][2] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.radianAngularVelocity);
                EntryReal.set(Zq, derivatives[2][2] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.radianAngularVelocity);
                EntryReal.set(Lq, derivatives[3][2] / IxxValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.radianAngularVelocity);
                EntryReal.set(Mq, derivatives[4][2] / IyyValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.radianAngularVelocity);
                EntryReal.set(Nq, derivatives[5][2] / IzzValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Qq, derivatives[6][2] / IRValue, "%6.2e", Units.UnitsList.rpmAcceleration,
                        Units.UnitsList.radianAngularVelocity);

                EntryReal.set(Xp, derivatives[0][3] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.radianAngularVelocity);
                EntryReal.set(Yp, derivatives[1][3] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.radianAngularVelocity);
                EntryReal.set(Zp, derivatives[2][3] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.radianAngularVelocity);
                EntryReal.set(Lp, derivatives[3][3] / IxxValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.radianAngularVelocity);
                EntryReal.set(Mp, derivatives[4][3] / IyyValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.radianAngularVelocity);
                EntryReal.set(Np, derivatives[5][3] / IzzValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.radianAngularVelocity);
                EntryReal.set(Qp, derivatives[6][3] / IRValue, "%6.2e", Units.UnitsList.rpmAcceleration,
                        Units.UnitsList.radianAngularVelocity);

                EntryReal.set(XW, derivatives[0][4] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.rpm);
                EntryReal.set(YW, derivatives[1][4] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.rpm);
                EntryReal.set(ZW, derivatives[2][4] / mValue, "%6.2e", Units.UnitsList.acceleration,
                        Units.UnitsList.rpm);
                EntryReal.set(LW, derivatives[3][4] / IxxValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.rpm);
                EntryReal.set(MW, derivatives[4][4] / IyyValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.rpm);
                EntryReal.set(NW, derivatives[5][4] / IzzValue, "%6.2e", Units.UnitsList.radianAngularAcceleration,
                        Units.UnitsList.rpm);
                EntryReal.set(QW, derivatives[6][4] / IRValue, "%6.2e", Units.UnitsList.rpmAcceleration,
                        Units.UnitsList.rpm);

            } else {
                EntryReal.set(Xu, derivatives[0][0], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Yu, derivatives[1][0], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Zu, derivatives[2][0], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Lu, derivatives[3][0], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.velocity2);
                EntryReal.set(Mu, derivatives[4][0], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.velocity2);
                EntryReal.set(Nu, derivatives[5][0], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.velocity2);
                EntryReal.set(Qu, derivatives[6][0], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.velocity2);

                EntryReal.set(Xw, derivatives[0][1], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Yw, derivatives[1][1], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Zw, derivatives[2][1], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.velocity2);
                EntryReal.set(Lw, derivatives[3][1], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.velocity2);
                EntryReal.set(Mw, derivatives[4][1], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.velocity2);
                EntryReal.set(Nw, derivatives[5][1], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.velocity2);
                EntryReal.set(Qw, derivatives[6][1], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.velocity2);

                EntryReal.set(Xq, derivatives[0][2], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Yq, derivatives[1][2], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Zq, derivatives[2][2], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Lq, derivatives[3][2], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Mq, derivatives[4][2], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Nq, derivatives[5][2], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Qq, derivatives[6][2], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.angularVelocity);

                EntryReal.set(Xp, derivatives[0][3], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Yp, derivatives[1][3], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Zp, derivatives[2][3], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Lp, derivatives[3][3], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Mp, derivatives[4][3], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Np, derivatives[5][3], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.angularVelocity);
                EntryReal.set(Qp, derivatives[6][3], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.angularVelocity);

                EntryReal.set(XW, derivatives[0][4], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.rpm);
                EntryReal.set(YW, derivatives[1][4], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.rpm);
                EntryReal.set(ZW, derivatives[2][4], "%6.2e", Units.UnitsList.force,
                        Units.UnitsList.rpm);
                EntryReal.set(LW, derivatives[3][4], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.rpm);
                EntryReal.set(MW, derivatives[4][4], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.rpm);
                EntryReal.set(NW, derivatives[5][4], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.rpm);
                EntryReal.set(QW, derivatives[6][4], "%6.2e", Units.UnitsList.moment,
                        Units.UnitsList.rpm);
            }
        }
    }
}
