/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors  have only  limited
 * liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and, more generally,to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package main.atmosphere;

/**
 * Atmosphere interface used to extends all atmosphere models.
 *
 * @author J. Fourcade
 */
public abstract class Atmosphere {

    /**
     *
     * @return air temperature (°C)
     */
    public abstract double getTemperature();

    /**
     *
     * @return air pressure (Pa)
     */
    public abstract double getPressure();

    /**
     *
     * @return air density (kg/M3)
     */
    public abstract double getDensity();

    /**
     *
     * @return brake power altitude factor
     */
    public abstract double getBrakePowerAltitudeFactor();

    /**
     *
     * @return speed of sound (m/s)
     */
    public abstract double getSpeedOfSound();

    /**
     *
     * @return acceleration of gravity (m/s2)
     */
    public abstract double getAccelerationOfGravity();

    /**
     *
     * @return kinetic viscosity (kg/m/s)
     */
    public abstract double getKinematicViscosity();

    /**
     *
     * @return atmosphere name
     */
    public abstract String getName();

    /**
     * Print on standard output atmosphere parameters.
     */
    public void print() {
        System.out.println("Atmopshere");
        System.out.println("----------");
        System.out.println("Temperature (°C)               : " + getTemperature());
        System.out.println("Pressure (Pa)                  : " + getPressure());
        System.out.println("Density (kg/m3)                : " + getDensity());
        System.out.println("Brake power altitude factor    : " + getBrakePowerAltitudeFactor());
        System.out.println("Speed of sound (m/s)           : " + getSpeedOfSound());
        System.out.println("Acceleration of gravity (m/s2) : " + getAccelerationOfGravity());
        System.out.println("Kinematic viscosity (kg/m/s)   : " + getKinematicViscosity());
    }
}
