/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package main.rotor;

import org.apache.commons.math3.util.FastMath;

/**
 * Normalized rotor for induced velocity testing (thrust velocity is one).
 *
 * @author J. Fourcade
 */
public class NormalizedRotor implements TrustVelocitylRotor {

    double U;
    double alphaS;

    /**
     * Single constructor. Compute velocity and no-feathering plane incidence.
     *
     * @param barMu Normaized advance velocity value.
     * @param barLambda Normalized vertical velocity value.
     */
    public NormalizedRotor(double barMu, double barLambda) {
        U = Math.sqrt(Math.pow(barMu, 2) + Math.pow(barLambda, 2));
        alphaS = Math.atan2(-barLambda, barMu);
    }

    
    @Override
    public double computeTrustVelocity(double vi) {
        return 1;
    }

    @Override
    public double getTangentialVelocity() {
        return U*FastMath.cos(alphaS);
    }

    @Override
    public double getNormalVelocity() {
         return U*FastMath.sin(alphaS);
    }

    
}
