/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package main.utils;

import org.apache.commons.math3.geometry.euclidean.threed.Vector3D;

/**
 * Torsor class to define pair of force and moment
 * 
 * @author J. Fourcade
 */
public class Torsor {
    
    final private Vector3D force ;
    final private Vector3D moment ;
    
    /**
     * Define a null torsor (force and moment vectors are set to zero)
     */
    public Torsor() {
        force = new Vector3D(0,0,0) ;
        moment = new Vector3D(0,0,0) ;
    }
    
    /**
     * Define a torsor with a given force and moment vectors
     * 
     * @param force : force vector (N)
     * @param moment : moment vector (N.m)
     */
    public Torsor(Vector3D force, Vector3D moment) {
        this.force = force ;
        this.moment = moment ;
    }
    
    /**
     * Get force vector
     * 
     * @return force vector (N)
     */
    public Vector3D getForce() {
        return force ;
    }
    
    /**
     * Get moment vector
     * 
     * @return moment vector (N.m)
     */
    public Vector3D getMoment() {
        return moment ;
    }

    /**
     * Add a torsor to this torsor.
     * 
     * @param torsor added torsor
     * @return sums of torsor
     */
    public Torsor add(Torsor torsor) {
        return new Torsor(getForce().add(torsor.getForce()),
                getMoment().add(torsor.getMoment())) ;
    }
    
    @Override
    public String toString() {
        return "F: "+force.toString()+"\nM: "+moment.toString();
    }
}
