/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr\
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info\
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package gyrorotor;

import gui.Utils.EntryReal;
import java.awt.Color;
import java.awt.Font;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import main.utils.Units;
import net.miginfocom.swing.MigLayout;

/**
 * Blade angle of attack distribution panel
 * 
 * @author J. Fourcade
 */
public class AoaDistributionData extends JPanel {

    // Datas
    final private JTextField[] aoa;
    static final Color[] colors = {Color.magenta, Color.cyan, Color.BLUE, Color.GREEN,
        Color.YELLOW, Color.ORANGE, Color.RED};

    /**
     * Constructor
     */
    public AoaDistributionData() {

        // Set layout
        setLayout(new MigLayout("insets 5 5 5 5"));

        // Panel font and title font
        setFont(new Font("Courier", Font.PLAIN, 12));
        Font font = getFont().deriveFont(Font.BOLD, 16);

        // Title
        JLabel title = new JLabel("Aoa values :");
        title.setFont(font);
        this.add(title);

        // Values
        aoa = new JTextField[7];
        double val = 2;
        for (int i = 0; i < 7; i++) {
            aoa[i] = new JTextField(String.format("%.1f", val));
            aoa[i].setForeground(colors[i]);
            add(aoa[i], "w 50");
            val = val + 2;
        }

        // Unit
        JLabel aoaUnit = new JLabel(Units.getLabel(Units.UnitsList.angle));
        add(aoaUnit);
    }

    /**
     * Get aoa values enetered by user.
     * 
     * @return aoa values
     */
    public double[] getAoaValues() {
        final double[] values = new double[7];
        for (int i = 0; i < 7; i++) {
            values[i] = EntryReal.read(aoa[i], Units.UnitsList.angle);
        }
        return values;
    }
    
    /**
     * Get aoa colors
     * 
     * @return aoa colors
     */
    public static Color[] getAoaColors() {
        return colors;
    }
}
